==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/my_syslog.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: my_syslog.c,v 1.7 2003/01/02 16:48:43 lwa Exp $";

#include "config.h"

#if HAVE_GETADDRINFO

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <syslog.h>
#include <netdb.h>
#include <inttypes.h>

#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "lmtp.h"


int my_syslog_check_addr(struct fstate *state, int af, void *src) {
  char printable[1024];
  char *s;

  if (inet_ntop(af, src, printable, sizeof(printable))==NULL) {
    syslog(LOG_NOTICE, "syslog(%s): illegal address format", 
	   state->rcpt->mbox);
    return -1;
  }

  switch(af) {
  case AF_INET:
    { 
      uint32_t addr =  ntohl(*(uint32_t*)src);
      if (IN_MULTICAST(addr) || IN_BADCLASS(addr))  {
	syslog(LOG_NOTICE, "syslog(%s): illegal address class for %s",
	       state->rcpt->mbox, printable);
	return -1;
      }
    }
    break;
  default:
    syslog(LOG_NOTICE, "syslog(%s), illegal address family %d", 
	   state->rcpt->mbox, af);
    return -1;
  }
  s = printable + strlen(printable);
  while(s > printable) {
    char *ret = slist_get_value(&(state->syslog_acl), printable);
    if (ret) {
      if (*ret == 0) 
	return 0;
      syslog(LOG_NOTICE, "syslog(%s): disallow syslog for addr %s: %s",
	     state->rcpt->mbox, printable, ret);
      return -1;
    }
    do {
      *s--=0;
    } while(s > printable && *s != '.' && *s != ':');
  }
  return 0;
}

int my_syslog_check_name(struct fstate *state, char *name) {
  do {
    char *ret = slist_get_value(&(state->syslog_acl), name);
    if (ret) {
      if (*ret == 0) return 0;
      syslog(LOG_NOTICE, "syslog(%s): disallow syslog for name %s: %s",
	     state->rcpt->mbox, name, ret);
      return -1;
    }
    do {
      name ++;
    } while(*name && (isalnum((int)*name) || *name == '-'));

    if (name[0] && (name[0] != '.' || name[1] == 0 || name[1] == '.')) {
      syslog(LOG_NOTICE, "syslog(%s): invalid syslog name '%s'",
	     state->rcpt->mbox, name);
      return -1;
    }
  } while(*name);
  return 0;
}

int my_syslog_check_user(struct fstate *state, char *user) {
  if (user) {
    char *ret = slist_get_value(&(state->syslog_acl), user);
    if (ret) {
      if (*ret == 0) return 0;
      syslog(LOG_NOTICE, "syslog(%s): disallow syslog for %s: %s",
	     state->rcpt->mbox, user, ret);
      return -1;
    }
    while(*user && *user != '@')
      user ++;
    if (*user=='@') {
      ret = slist_get_value(&(state->syslog_acl), user);
      if (ret) {
	if (*ret == 0) return 0;
	syslog(LOG_NOTICE, "syslog(%s): disallow syslog for %s: %s",
	       state->rcpt->mbox, user, ret);
	return -1;
      }
    }
  }
  return 0;
}

int my_syslog_check(struct fstate *state, char *host) {
  if (host) {
    char *s;

    if (strchr(host, '.') == NULL) {
      syslog(LOG_NOTICE, "syslog(%s): illegal syslog host name '%s'",
	     state->rcpt->mbox, host);
      return -1;
    }
    s = host + strspn(host, "0123456789.");

    if (state->rcpt && my_syslog_check_user(state, state->rcpt->mbox) == -1)
      return -1;

    if (*s == 0) {
      char m[1024];
      if (inet_pton(AF_INET, host, &m) != 1) {
	syslog(LOG_NOTICE, "syslog(%s): illegal address format %s",
	       state->rcpt->mbox, host);
	return -1;
      }
      return my_syslog_check_addr(state, AF_INET, m);
    } else {
      return my_syslog_check_name(state, host);
    }
  }
  return 0;
}

void my_syslog(struct fstate *state, char *fmt, ...) {
  if (state->syslog_host) {
    int error;
    struct addrinfo hints, *res;
    struct user_res_ctx res_ctx;

    memset(&hints, 0, sizeof(hints));
    hints.ai_family = PF_INET;
    hints.ai_socktype = SOCK_DGRAM;

    user_res_begin(state, &res_ctx);
    error = getaddrinfo(state->syslog_host, "514", &hints, &res);
    user_res_end(state, &res_ctx);

    if (error) {
      syslog(LOG_NOTICE, "syslog(%s): %s: %s",
	     state->rcpt->mbox, state->syslog_host, gai_strerror(error));
    } else {
      va_list ap;
      struct addrinfo *r = res;
      char buf[512];
      int buflen=sprintf(buf, "<%d>", LOG_MAIL|LOG_INFO);
      va_start(ap, fmt);
      buflen += vsnprintf(buf+buflen, sizeof(buf)-buflen, fmt, ap);
      va_end(ap);
      if (buflen > sizeof(buf))
	buflen = sizeof(buf);
      while(r) {
	int s;
	switch(r->ai_family) {
	case AF_INET:
	  if (my_syslog_check_addr(state, r->ai_family,
				   &(((struct sockaddr_in*)r->ai_addr)
				     ->sin_addr)  ) ==-1)
	    return;
	  break;
	}
	s = socket(r->ai_family, r->ai_socktype, r->ai_protocol);
	if (s>0) {
	  int lsent;
	  switch(r->ai_family) {
	  case AF_INET: 
	    if (state->syslog_src) {
	      socklen_t sin_len;
#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
	      sin_len = state->syslog_src->sin_len;
#else
	      sin_len = sizeof(struct sockaddr);
#endif
	      bind(s, (struct sockaddr*)(state->syslog_src), sin_len);
	    }
	    break;
	  }
	  lsent = sendto(s, buf, buflen, 0,
			 r->ai_addr, r->ai_addrlen);

	  close(s);
	  if (lsent >0)
	    break;
	}
	r = r->ai_next;
      }
      freeaddrinfo(res);
    }
  }
}

#else 
/* no getaddrinfo (hpux) */

#include "lmtp.h"
void my_syslog(struct fstate *state, char *fmt, ...) {
  return;
}
int my_syslog_check(struct fstate *state, char *name) {
  return -1;
}
#endif

==> lmtpd-0.10.7.tar.gz: lmtpd-0.10.7/my_syslog.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: my_syslog.c,v 1.8 2004/10/26 09:59:20 lwa Exp $";



#if HAVE_GETADDRINFO

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <syslog.h>
#include <netdb.h>
#include <inttypes.h>

#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "lmtp.h"


int my_syslog_check_addr(struct fstate *state, int af, void *src) {
  char printable[1024];
  char *s;

  if (inet_ntop(af, src, printable, sizeof(printable))==NULL) {
    syslog(LOG_NOTICE, "syslog(%s): illegal address format", 
	   state->rcpt->mbox);
    return -1;
  }

  switch(af) {
  case AF_INET:
    { 
      uint32_t addr =  ntohl(*(uint32_t*)src);
      if (IN_MULTICAST(addr) || IN_BADCLASS(addr))  {
	syslog(LOG_NOTICE, "syslog(%s): illegal address class for %s",
	       state->rcpt->mbox, printable);
	return -1;
      }
    }
    break;
  default:
    syslog(LOG_NOTICE, "syslog(%s), illegal address family %d", 
	   state->rcpt->mbox, af);
    return -1;
  }
  s = printable + strlen(printable);
  while(s > printable) {
    char *ret = slist_get_value(&(state->syslog_acl), printable);
    if (ret) {
      if (*ret == 0) 
	return 0;
      syslog(LOG_NOTICE, "syslog(%s): disallow syslog for addr %s: %s",
	     state->rcpt->mbox, printable, ret);
      return -1;
    }
    do {
      *s--=0;
    } while(s > printable && *s != '.' && *s != ':');
  }
  return 0;
}

int my_syslog_check_name(struct fstate *state, char *name) {
  do {
    char *ret = slist_get_value(&(state->syslog_acl), name);
    if (ret) {
      if (*ret == 0) return 0;
      syslog(LOG_NOTICE, "syslog(%s): disallow syslog for name %s: %s",
	     state->rcpt->mbox, name, ret);
      return -1;
    }
    do {
      name ++;
    } while(*name && (isalnum((int)*name) || *name == '-'));

    if (name[0] && (name[0] != '.' || name[1] == 0 || name[1] == '.')) {
      syslog(LOG_NOTICE, "syslog(%s): invalid syslog name '%s'",
	     state->rcpt->mbox, name);
      return -1;
    }
  } while(*name);
  return 0;
}

int my_syslog_check_user(struct fstate *state, char *user) {
  if (user) {
    char *ret = slist_get_value(&(state->syslog_acl), user);
    if (ret) {
      if (*ret == 0) return 0;
      syslog(LOG_NOTICE, "syslog(%s): disallow syslog for %s: %s",
	     state->rcpt->mbox, user, ret);
      return -1;
    }
    while(*user && *user != '@')
      user ++;
    if (*user=='@') {
      ret = slist_get_value(&(state->syslog_acl), user);
      if (ret) {
	if (*ret == 0) return 0;
	syslog(LOG_NOTICE, "syslog(%s): disallow syslog for %s: %s",
	       state->rcpt->mbox, user, ret);
	return -1;
      }
    }
  }
  return 0;
}

int my_syslog_check(struct fstate *state, char *host) {
  if (host) {
    char *s;

    if (strchr(host, '.') == NULL) {
      syslog(LOG_NOTICE, "syslog(%s): illegal syslog host name '%s'",
	     state->rcpt->mbox, host);
      return -1;
    }
    s = host + strspn(host, "0123456789.");

    if (state->rcpt && my_syslog_check_user(state, state->rcpt->mbox) == -1)
      return -1;

    if (*s == 0) {
      char m[1024];
      if (inet_pton(AF_INET, host, &m) != 1) {
	syslog(LOG_NOTICE, "syslog(%s): illegal address format %s",
	       state->rcpt->mbox, host);
	return -1;
      }
      return my_syslog_check_addr(state, AF_INET, m);
    } else {
      return my_syslog_check_name(state, host);
    }
  }
  return 0;
}

void my_syslog(struct fstate *state, char *fmt, ...) {
  if (state->syslog_host) {
    int error;
    struct addrinfo hints, *res;
    struct user_res_ctx res_ctx;

    memset(&hints, 0, sizeof(hints));
    hints.ai_family = PF_INET;
    hints.ai_socktype = SOCK_DGRAM;

    user_res_begin(state, &res_ctx);
    error = getaddrinfo(state->syslog_host, "514", &hints, &res);
    user_res_end(state, &res_ctx);

    if (error) {
      syslog(LOG_NOTICE, "syslog(%s): %s: %s",
	     state->rcpt->mbox, state->syslog_host, gai_strerror(error));
    } else {
      va_list ap;
      struct addrinfo *r = res;
      char buf[512];
      int buflen=sprintf(buf, "<%d>", LOG_MAIL|LOG_INFO);
      va_start(ap, fmt);
      buflen += vsnprintf(buf+buflen, sizeof(buf)-buflen, fmt, ap);
      va_end(ap);
      if (buflen > sizeof(buf))
	buflen = sizeof(buf);
      while(r) {
	int s;
	switch(r->ai_family) {
	case AF_INET:
	  if (my_syslog_check_addr(state, r->ai_family,
				   &(((struct sockaddr_in*)r->ai_addr)
				     ->sin_addr)  ) ==-1)
	    return;
	  break;
	}
	s = socket(r->ai_family, r->ai_socktype, r->ai_protocol);
	if (s>0) {
	  int lsent;
	  switch(r->ai_family) {
	  case AF_INET: 
	    if (state->syslog_src) {
	      socklen_t sin_len;
#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
	      sin_len = state->syslog_src->sin_len;
#else
	      sin_len = sizeof(struct sockaddr);
#endif
	      bind(s, (struct sockaddr*)(state->syslog_src), sin_len);
	    }
	    break;
	  }
	  lsent = sendto(s, buf, buflen, 0,
			 r->ai_addr, r->ai_addrlen);

	  close(s);
	  if (lsent >0)
	    break;
	}
	r = r->ai_next;
      }
      freeaddrinfo(res);
    }
  }
}

#else 
/* no getaddrinfo (hpux) */

#include "lmtp.h"
void my_syslog(struct fstate *state, char *fmt, ...) {
  return;
}
int my_syslog_check(struct fstate *state, char *name) {
  return -1;
}
#endif
